@@ -104,7 +104,7 @@ static void connection_or_check_canonicity(or_connection_t *conn,
 /**
  * Cast a `connection_t *` to an `or_connection_t *`.
  *
- * Exit with an assertion failure if the input is not an `or_connnection_t`.
+ * Exit with an assertion failure if the input is not an `or_connection_t`.
  **/
 or_connection_t *
 TO_OR_CONN(connection_t *c)
@@ -116,7 +116,7 @@ TO_OR_CONN(connection_t *c)
 /**
  * Cast a `const connection_t *` to a `const or_connection_t *`.
  *
- * Exit with an assertion failure if the input is not an `or_connnection_t`.
+ * Exit with an assertion failure if the input is not an `or_connection_t`.
  **/
 const or_connection_t *
 CONST_TO_OR_CONN(const connection_t *c)
@@ -179,13 +179,18 @@ connection_or_set_identity_digest(or_connection_t *conn,
   const int rsa_id_was_set = ! tor_digest_is_zero(conn->identity_digest);
   const int ed_id_was_set =
     chan && !ed25519_public_key_is_zero(&chan->ed25519_identity);
+  const int new_ed_id_is_set =
+    (ed_id && !ed25519_public_key_is_zero(ed_id));
   const int rsa_changed =
     tor_memneq(conn->identity_digest, rsa_digest, DIGEST_LEN);
-  const int ed_changed = ed_id_was_set &&
-    (!ed_id || !ed25519_pubkey_eq(ed_id, &chan->ed25519_identity));
+  const int ed_changed = bool_neq(ed_id_was_set, new_ed_id_is_set) ||
+    (ed_id_was_set && new_ed_id_is_set && chan &&
+     !ed25519_pubkey_eq(ed_id, &chan->ed25519_identity));
 
-  tor_assert(!rsa_changed || !rsa_id_was_set);
-  tor_assert(!ed_changed || !ed_id_was_set);
+  if (BUG(rsa_changed && rsa_id_was_set))
+    return;
+  if (BUG(ed_changed && ed_id_was_set))
+    return;
 
   if (!rsa_changed && !ed_changed)
     return;
@@ -200,8 +205,7 @@ connection_or_set_identity_digest(or_connection_t *conn,
   memcpy(conn->identity_digest, rsa_digest, DIGEST_LEN);
 
   /* If we're initializing the IDs to zero, don't add a mapping yet. */
-  if (tor_digest_is_zero(rsa_digest) &&
-      (!ed_id || ed25519_public_key_is_zero(ed_id)))
+  if (tor_digest_is_zero(rsa_digest) && !new_ed_id_is_set)
     return;
 
   /* Deal with channels */
@@ -806,6 +810,10 @@ connection_or_about_to_close(or_connection_t *or_conn)
   } else if (!tor_digest_is_zero(or_conn->identity_digest)) {
     connection_or_event_status(or_conn, OR_CONN_EVENT_CLOSED,
                 tls_error_to_orconn_end_reason(or_conn->tls_error));
+  } else {
+    /* Normal close, we notify of a done connection. */
+    connection_or_event_status(or_conn, OR_CONN_EVENT_CLOSED,
+                               END_OR_CONN_REASON_DONE);
   }
 }
 
@@ -1072,7 +1080,7 @@ connection_or_group_set_badness_(smartlist_t *group, int force)
    * XXXX connections. */
 
   or_connection_t *best = NULL;
-  int n_old = 0, n_inprogress = 0, n_canonical = 0, n_other = 0;
+  int n_canonical = 0;
   time_t now = time(NULL);
 
   /* Pass 1: expire everything that's old, and see what the status of
@@ -1081,14 +1089,8 @@ connection_or_group_set_badness_(smartlist_t *group, int force)
     if (connection_or_single_set_badness_(now, or_conn, force))
       continue;
 
-    if (connection_or_is_bad_for_new_circs(or_conn)) {
-      ++n_old;
-    } else if (or_conn->base_.state != OR_CONN_STATE_OPEN) {
-      ++n_inprogress;
-    } else if (or_conn->is_canonical) {
+    if (or_conn->is_canonical) {
       ++n_canonical;
-    } else {
-      ++n_other;
     }
   } SMARTLIST_FOREACH_END(or_conn);
 
@@ -1316,6 +1318,13 @@ note_or_connect_failed(const or_connection_t *or_conn)
 
   tor_assert(or_conn);
 
+  if (or_conn->potentially_used_for_bootstrapping) {
+    /* Don't cache connection failures for connections we initiated ourself.
+     * If these direct connections fail, we're supposed to recognize that
+     * the destination is down and stop trying. See ticket 40499. */
+    return;
+  }
+
   ocf = or_connect_failure_find(or_conn);
   if (ocf == NULL) {
     ocf = or_connect_failure_new(or_conn);
