@@ -1,7 +1,7 @@
 /* Copyright (c) 2001 Matej Pfajfar.
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
- * Copyright (c) 2007-2020, The Tor Project, Inc. */
+ * Copyright (c) 2007-2021, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -35,7 +35,9 @@ node_t *nodelist_add_microdesc(microdesc_t *md);
 void nodelist_set_consensus(const networkstatus_t *ns);
 void nodelist_ensure_freshness(const networkstatus_t *ns);
 int nodelist_probably_contains_address(const tor_addr_t *addr);
-void nodelist_add_addr_to_address_set(const tor_addr_t *addr);
+bool nodelist_reentry_contains(const tor_addr_t *addr, uint16_t port);
+void nodelist_add_addr_to_address_set(const tor_addr_t *addr,
+                                      uint16_t or_port, uint16_t dir_port);
 
 void nodelist_remove_microdesc(const char *identity_digest, microdesc_t *md);
 void nodelist_remove_routerinfo(routerinfo_t *ri);
@@ -82,6 +84,7 @@ bool node_supports_establish_intro_dos_extension(const node_t *node);
 bool node_supports_initiating_ipv6_extends(const node_t *node);
 bool node_supports_accepting_ipv6_extends(const node_t *node,
                                           bool need_canonical_ipv6_conn);
+bool node_supports_conflux(const node_t *node);
 
 const uint8_t *node_get_rsa_id_digest(const node_t *node);
 MOCK_DECL(smartlist_t *,node_get_link_specifier_smartlist,(const node_t *node,
@@ -106,7 +109,6 @@ void node_get_pref_ipv6_dirport(const node_t *node, tor_addr_port_t *ap_out);
 int node_has_curve25519_onion_key(const node_t *node);
 const struct curve25519_public_key_t *node_get_curve25519_onion_key(
                                   const node_t *node);
-crypto_pk_t *node_get_rsa_onion_key(const node_t *node);
 
 MOCK_DECL(const smartlist_t *, nodelist_get_list, (void));
 
